# -*- coding: utf-8 -*-

"""
py_vollib.black.greeks.analytical
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from __future__ import division

# Related third party imports
import numpy

# Local application/library specific imports
from py_lets_be_rational import norm_cdf as N
from py_vollib.helpers import pdf
from py_vollib.black import black
from py_vollib.ref_python.black import d1, d2


# -----------------------------------------------------------------------------
# FUNCTIONS - ANALYTICAL GREEKS

def delta(flag, F, K, t, r, sigma):
    """Returns the Black delta of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float
    
    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = delta(flag, F, K, t, r, sigma)
    >>> v2 = 0.45107017482201828
    >>> abs(v1-v2) < .000001
    True
    """

    D1 = d1(F, K, t, r, sigma)

    if flag == 'p':
        return - numpy.exp(-r*t) * N(-D1)
    else:
        return numpy.exp(-r*t) * N(D1)


def theta(flag, F, K, t, r, sigma):
    """Returns the Black theta of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 

    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = theta(flag, F, K, t, r, sigma)
    >>> v2 = -0.00816236877462
    >>> abs(v1-v2) < .000001
    True
    >>> flag = 'p'
    >>> v1 = theta(flag, F, K, t, r, sigma)
    >>> v2 = -0.00802799155312
    >>> abs(v1-v2) < .000001
    True
    """

    e_to_the_minus_rt = numpy.exp(-r*t)
    two_sqrt_t = 2 * numpy.sqrt(t)

    D1 = d1(F, K, t, r, sigma)
    D2 = d2(F, K, t, r, sigma)
    pdf_d1 = pdf(D1)
    N_d2 = N(D2)

    first_term = F * e_to_the_minus_rt * pdf(D1) * sigma / two_sqrt_t 

    if flag == 'c':        
        second_term = -r * F * e_to_the_minus_rt * N(D1)
        third_term = r * K * e_to_the_minus_rt * N(D2)
        return -(first_term + second_term + third_term) / 365.
    else:
        second_term = -r * F * e_to_the_minus_rt * N(-D1)
        third_term = r * K * e_to_the_minus_rt * N(-D2)
        return (-first_term + second_term + third_term) / 365.0


def gamma(flag, F, K, t, r, sigma):
    """Returns the Black gamma of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 

    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = gamma(flag, F, K, t, r, sigma)
    >>> # 0.0640646705882
    >>> v2 = 0.0640646705882
    >>> abs(v1-v2) < .000001
    True
    """

    D1 = d1(F, K, t, r, sigma)
    return pdf(D1)*numpy.exp(-r*t)/(F*sigma*numpy.sqrt(t))


def vega(flag, F, K, t, r, sigma):
    """Returns the Black vega of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float     
    
    ::
    
      ==========================================================
      Note: The text book analytical formula does not multiply by .01,
      but in practice vega is defined as the change in price
      for each 1 percent change in IV, hence we multiply by 0.01.
      ==========================================================
    
    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = vega(flag, F, K, t, r, sigma)
    >>> # 0.118317785624
    >>> v2 = 0.118317785624
    >>> abs(v1-v2) < .000001
    True
    """

    D1 = d1(F, K, t, r, sigma)
    return F * numpy.exp(-r*t) * pdf(D1) * numpy.sqrt(t) * 0.01


def rho(flag, F, K, t, r, sigma):
    """Returns the Black rho of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float     

    ::

      ==========================================================
      The text book analytical formula does not multiply by .01,
      but in practice rho is defined as the change in price
      for each 1 percent change in r, hence we multiply by 0.01.
      ==========================================================
      
    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = rho(flag, F, K, t, r, sigma)
    >>> v2 = -0.0074705380059582258
    >>> abs(v1-v2) < .000001
    True
    >>> flag = 'p'
    >>> v1 = rho(flag, F, K, t, r, sigma)
    >>> v2 = -0.011243286001308292
    >>> abs(v1-v2) < .000001
    True
    """

    return -t * black(flag, F, K, t, r, sigma) * .01


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()



# -*- coding: utf-8 -*-

"""
py_vollib.black_scholes_merton
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports

# Related third party imports
import numpy

# Local application/library specific imports
from py_lets_be_rational import black
from py_vollib.helpers import binary_flag


# -----------------------------------------------------------------------------
# FUNCTIONS

def black_scholes_merton(flag, S, K, t, r, sigma, q):
    """Return the Black-Scholes-Merton option price.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float 

    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2

    >>> p_published_value = 2.4648
    >>> p_calc = black_scholes_merton('p', S, K, t, r, sigma, q)
    >>> abs(p_published_value - p_calc) < 0.0001
    True
    """
    
    F = S * numpy.exp((r-q)*t)
    deflater = numpy.exp(-r * t)
    return black(F, K, sigma, t, binary_flag[flag]) * deflater


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()



# -*- coding: utf-8 -*-

"""
py_vollib.ref_python.black_scholes_merton
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from __future__ import division

# Related third party imports
import numpy
from scipy.stats import norm

# Local application/library specific imports


N = norm.cdf


# -----------------------------------------------------------------------------
# FUNCTIONS, FOR REFERENCE AND TESTING

def d1(S, K, t, r, sigma, q):
    """Calculate the d1 component of the Black-Scholes-Merton PDE.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float

    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2

    >>> d1_published_value = 0.6102
    >>> d1_calc = d1(S,K,t,r,sigma,q)
    >>> abs(d1_published_value - d1_calc) < 0.0001
    True
    """

    numerator = numpy.log(S / float(K)) + ((r - q) + sigma * sigma / 2.0) * t
    denominator = sigma * numpy.sqrt(t)
    return numerator / denominator


def d2(S, K, t, r, sigma, q):
    """Calculate the d2 component of the Black-Scholes-Merton PDE.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float

    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2

    >>> d2_published_value = 0.4688
    >>> d2_calc = d2(S,K,t,r,sigma,q)
    >>> abs(d2_published_value - d2_calc) < 0.0001
    True
    """

    return d1(S, K, t, r, sigma, q) - sigma * numpy.sqrt(t)


def bsm_call(S, K, t, r, sigma, q):
    """Return the Black-Scholes-Merton call price
    implemented in python (for reference).

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float
    """

    D1 = d1(S, K, t, r, sigma, q)
    D2 = d2(S, K, t, r, sigma, q)

    return S * numpy.exp(-q * t) * N(D1) - K * numpy.exp(-r * t) * N(D2)


def bsm_put(S, K, t, r, sigma, q):
    """Return the Black-Scholes-Merton put price
    implemented in python (for reference).

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float

    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2

    >>> p_published_value = 2.4648
    >>> p_calc = bsm_put(S, K, t, r, sigma, q)
    >>> abs(p_published_value - p_calc) < 0.0001
    True
    """

    D1 = d1(S, K, t, r, sigma, q)
    D2 = d2(S, K, t, r, sigma, q)
    return K * numpy.exp(-r * t) * N(-D2) - S * numpy.exp(-q * t) * N(-D1)


def black_scholes_merton(flag, S, K, t, r, sigma, q):
    """Return the Black-Scholes-Merton call price implemented in
    python (for reference).

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str

    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2

    >>> p_published_value = 2.4648
    >>> p_calc = black_scholes_merton('p', S, K, t, r, sigma, q)
    >>> abs(p_published_value - p_calc) < 0.0001
    True
    """

    if flag == 'c':
        return bsm_call(S, K, t, r, sigma, q)
    else:
        return bsm_put(S, K, t, r, sigma, q)


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()



# -*- coding: utf-8 -*-

"""
py_vollib.ref_python.black_scholes_merton.greeks.analytical
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from __future__ import division

# Related third party imports
import numpy
from scipy.stats import norm

# Local application/library specific imports
from py_vollib.helpers import pdf
from py_vollib.ref_python.black_scholes_merton import d1, d2


N = norm.cdf


# -----------------------------------------------------------------------------
# FUNCTIONS - ANALYTICAL GREEKS

def delta(flag, S, K, t, r, sigma, q):
    """Returns the Black-Scholes-Merton delta of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float

    Example 17.1, page 355, Hull:

    >>> S = 49
    >>> K = 50
    >>> r = .05
    >>> t = 0.3846
    >>> q = 0
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> delta_calc = delta(flag, S, K, t, r, sigma, q)
    >>> # 0.521601633972
    >>> delta_text_book = 0.522
    >>> abs(delta_calc - delta_text_book) < .01
    True
    """

    D1 = d1(S, K, t, r, sigma, q)

    if flag == 'p':
        return -numpy.exp(-q * t) * N(-D1)
    else:
        return numpy.exp(-q * t) * N(D1)


def theta(flag, S, K, t, r, sigma, q):
    """Returns the Black-Scholes-Merton theta of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float

    The text book analytical formula does not divide by 365,
    but in practice theta is defined as the change in price
    for each day change in t, hence we divide by 365.

    Example 17.2, page 359, Hull:

    >>> S = 49
    >>> K = 50
    >>> r = .05
    >>> t = 0.3846
    >>> q = 0
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> annual_theta_calc = theta(flag, S, K, t, r, sigma, q) * 365
    >>> # -4.30538996455
    >>> annual_theta_text_book = -4.31
    >>> abs(annual_theta_calc - annual_theta_text_book) < .01
    True

    Using the same inputs with a put.
    >>> S = 49
    >>> K = 50
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'p'
    >>> annual_theta_calc = theta(flag, S, K, t, r, sigma, q) * 365
    >>> # -1.8530056722
    >>> annual_theta_reference = -1.8530056722
    >>> abs(annual_theta_calc - annual_theta_reference) < .000001
    True
    """

    D1 = d1(S, K, t, r, sigma, q)
    D2 = d2(S, K, t, r, sigma, q)

    first_term = (S * numpy.exp(-q * t) * pdf(D1) * sigma) / (2 * numpy.sqrt(t))

    if flag == 'c':

        second_term = -q * S * numpy.exp(-q * t) * N(D1)
        third_term = r * K * numpy.exp(-r * t) * N(D2)

        return - (first_term + second_term + third_term) / 365.0

    else:

        second_term = -q * S * numpy.exp(-q * t) * N(-D1)
        third_term = r * K * numpy.exp(-r * t) * N(-D2)

        return (-first_term + second_term + third_term) / 365.0


def gamma(flag, S, K, t, r, sigma, q):
    """Returns the Black-Scholes-Merton gamma of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float

    Example 17.4, page 364, Hull:

    >>> S = 49
    >>> K = 50
    >>> r = .05
    >>> t = 0.3846
    >>> q = 0
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> gamma_calc = gamma(flag, S, K, t, r, sigma, q)
    >>> # 0.0655453772525
    >>> gamma_text_book = 0.066
    >>> abs(gamma_calc - gamma_text_book) < .001
    True
    """

    D1 = d1(S, K, t, r, sigma, q)
    numerator = numpy.exp(-q * t) * pdf(D1)
    denominator = S * sigma * numpy.sqrt(t)

    return numerator / denominator


def vega(flag, S, K, t, r, sigma, q):
    """Returns the Black-Scholes-Merton vega of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float

    The text book analytical formula does not multiply by .01,
    but in practice vega is defined as the change in price
    for each 1 percent change in IV, hence we multiply by 0.01.

    Example 17.6, page 367, Hull:

    >>> S = 49
    >>> K = 50
    >>> r = .05
    >>> t = 0.3846
    >>> q = 0
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> vega_calc = vega(flag, S, K, t, r, sigma, q)
    >>> # 0.121052427542
    >>> vega_text_book = 0.121
    >>> abs(vega_calc - vega_text_book) < .01
    True
    """

    D1 = d1(S, K, t, r, sigma, q)

    return S * numpy.exp(-q * t) * pdf(D1) * numpy.sqrt(t) * 0.01


def rho(flag, S, K, t, r, sigma, q):
    """Returns the Black-Scholes-Merton rho of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float

    The text book analytical formula does not multiply by .01,
    but in practice rho is defined as the change in price
    for each 1 percent change in r, hence we multiply by 0.01.

    Example 17.7, page 368, Hull:

    >>> S = 49
    >>> K = 50
    >>> r = .05
    >>> t = 0.3846
    >>> q = 0
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> rho_calc = rho(flag, S, K, t, r, sigma, q)
    >>> # 0.089065740988
    >>> rho_text_book = 0.0891
    >>> abs(rho_calc - rho_text_book) < .0001
    True
    """

    D2 = d2(S, K, t, r, sigma, q)

    if flag == 'c':
        return t * K * numpy.exp(-r * t) * N(D2) * .01
    else:
        return -t * K * numpy.exp(-r * t) * N(-D2) * .01


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()



# -*- coding: utf-8 -*-

"""
py_vollib.ref_python.black_scholes
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from __future__ import division

# Related third party imports
import numpy
from scipy.stats import norm

# Local application/library specific imports


N = norm.cdf


# -----------------------------------------------------------------------------
# FUNCTIONS - REFERENCE PYTHON IMPLEMENTATION, FOR COMPARISON

def d1(S, K, t, r, sigma):  # see Hull, page 292
    """Calculate the d1 component of the Black-Scholes PDE.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float

    John C. Hull, "Options, Futures and Other Derivatives,"
    7th edition, Example 13.6, page 294

    >>> S = 42
    >>> K = 40
    >>> r = .10
    >>> sigma = .20
    >>> t = 0.5
    >>> calculated_d1 = d1(S,K,t,r,sigma)
    >>> text_book_d1 = 0.7693
    >>> abs(calculated_d1 - text_book_d1) < 0.0001
    True
    """

    sigma_squared = sigma * sigma
    numerator = numpy.log(S / float(K)) + (r + sigma_squared / 2.) * t
    denominator = sigma * numpy.sqrt(t)

    if not denominator:
        print ('')
    return numerator / denominator


def d2(S, K, t, r, sigma):  # see Hull, page 292
    """Calculate the d2 component of the Black-Scholes PDE.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float

    John C. Hull, "Options, Futures and Other Derivatives,"
    7th edition, Example 13.6, page 294

    >>> S = 42
    >>> K = 40
    >>> r = .10
    >>> sigma = .20
    >>> t = 0.5
    >>> calculated_d2 = d2(S,K,t,r,sigma) #0.627841271869
    >>> text_book_d2 = 0.6278
    >>> abs(calculated_d2 - text_book_d2) < 0.0001
    True
    """

    return d1(S, K, t, r, sigma) - sigma * numpy.sqrt(t)


def black_scholes(flag, S, K, t, r, sigma):
    """Return the Black-Scholes option price implemented in
        python (for reference).

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str

    >>> S,K,t,r,sigma = 60,65,.25,.08,.3
    >>> expected = 2.13336844492
    >>> actual = black_scholes('c',S,K,t,r,sigma)
    >>> abs(expected-actual) < 1e-11
    True
    """

    e_to_the_minus_rt = numpy.exp(-r * t)
    D1 = d1(S, K, t, r, sigma)
    D2 = d2(S, K, t, r, sigma)
    if flag == 'c':
        return S * N(D1) - K * e_to_the_minus_rt * N(D2)
    else:
        return - S * N(-D1) + K * e_to_the_minus_rt * N(-D2)


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()



# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from collections import OrderedDict
import os

# Related third party imports
import simplejson as json
import pandas

# Local application/library specific imports


def almost_equal(a, b, epsilon=1.0e-7):
    return abs(a - b) < epsilon


class IteratorForTestData(object):

    """
    >>> data_iterator = IteratorForTestData()
    >>> print (data_iterator.has_next())
    True
    >>> r = data_iterator.next_row()
    >>> print (r['S'])
    100.0
    """
    
    def __init__(self):
        filepath = os.path.join(os.path.dirname(__file__), "test_data.json")
        self.data = json.load(open(filepath,'rb'))
        columns = self.data['columns']
        grid_data = OrderedDict()

        for header in columns:
            grid_data[header] = []

        for col in self.data['data']:
            for i in range(len(columns)):
                grid_data[columns[i]].append(col[i])

        self.df = pandas.DataFrame(grid_data)
        self.row_id = 0
        self.row_count = self.df.S.count()

    def next_row(self):
        if self.has_next():
            row = self.df.ix[self.row_id].to_dict()
            self.row_id += 1
            return row

    def has_next(self):
        return self.row_id < self.row_count
    
    
# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':  
    import doctest
    if not doctest.testmod().failed:
        print ("Doctest passed")



# -*- coding: utf-8 -*-

"""
py_vollib.ref_python.black
~~~~~~~~~~~~~~~~~~~~~~~~~~

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from __future__ import division
from math import log, sqrt

# Related third party imports
import numpy
from numpy import log, sqrt
from scipy.stats import norm

# Local application/library specific imports


N = norm.cdf


# -----------------------------------------------------------------------------
# FUNCTIONS - INTERNAL, FOR COMPARISON

'''
From John C. Hull, "Options, Futures and Other Derivatives,"
7th edition, Chapter 16.8, page 342
'''

def d1(F, K, t, r, sigma):  # keep r argument for consistency
    """Calculate the d1 component of the Black PDE.

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float

    Doctest using Hull, page 343, example 16.6

    >>> F = 20
    >>> K = 20
    >>> r = .09
    >>> t = 4/12.0
    >>> sigma = 0.25

    >>> calculated_value = d1(F, K, t, r, sigma)
    >>> #0.0721687836487
    >>> text_book_value = 0.07216
    >>> abs(calculated_value - text_book_value) < .00001
    True
    """

    sigma_squared = sigma * sigma
    numerator = log(F / float(K)) + sigma_squared * t / 2.0
    denominator = sigma * sqrt(t)

    return numerator / denominator


def d2(F, K, t, r, sigma):  # keep r argument for consistency
    """Calculate the d2 component of the Black PDE.

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float

    Hull, page 343, example 16.6

    >>> F = 20
    >>> K = 20
    >>> r = .09
    >>> t = 4/12.0
    >>> sigma = 0.25

    >>> calculated_value = d2(F, K, t, r, sigma)
    >>> #-0.0721687836487
    >>> text_book_value = -0.07216
    >>> abs(calculated_value - text_book_value) < .00001
    True
    """

    return d1(F, K, t, r, sigma) - sigma * sqrt(t)


def black_call(F, K, t, r, sigma):  # Equation 16.9
    """Calculate the price of a call using Black.  (Python
    implementation for reference.)

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float

    Hull, page 343, example 16.7

    >>> F = 620
    >>> K = 600
    >>> r = .05
    >>> sigma = .2
    >>> t = 0.5
    >>> calculated_value = black_call(F, K, t, r, sigma)
    >>> #44.1868533121
    >>> text_book_value = 44.19
    >>> abs(calculated_value - text_book_value) < .01
    True
    """

    deflater = numpy.exp(-r * t)
    N_d1 = N(d1(F, K, t, r, sigma))
    N_d2 = N(d2(F, K, t, r, sigma))

    return deflater * (F * N_d1 - K * N_d2)


def black_put(F, K, t, r, sigma):  # Equation 16.10
    """Calculate the price of a put using Black.  (Python
    implementation for reference.)

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float

    Hull, page 338, example 16.6

    >>> F = 20
    >>> K = 20
    >>> r = .09
    >>> sigma = .25
    >>> t = 4/12.0
    >>> calculated_value = black_put(F, K, t, r, sigma)
    >>> # 1.11664145656
    >>> text_book_value = 1.12
    >>> abs(calculated_value - text_book_value) < .01
    True
    """

    deflater = numpy.exp(-r * t)
    N_of_minus_d1 = N(-d1(F, K, t, r, sigma))
    N_of_minus_d2 = N(-d2(F, K, t, r, sigma))

    return deflater * (-F * N_of_minus_d1 + K * N_of_minus_d2)


def black(flag, F, K, t, r, sigma):
    """Calculate the (discounted) Black option price.

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float

    >>> F = 100
    >>> K = 100
    >>> sigma = .2
    >>> flag = 'c'
    >>> r = .02
    >>> t = .5

    >>> expected = 5.5811067246048118
    >>> actual = black(flag, F, K, t, r, sigma)
    >>> abs(expected - actual) < 1e-12
    True
    """

    if flag == 'c':
        return black_call(F, K, t, r, sigma)
    elif flag == 'p':
        return black_put(F, K, t, r, sigma)
    else:
        raise Exception("flag '{}' is invalid.".format(flag))


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()



# -*- coding: utf-8 -*-

"""
py_vollib.black_scholes.implied_volatility
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports

# Related third party imports
from py_lets_be_rational import implied_volatility_from_a_transformed_rational_guess as iv
import numpy

# Local application/library specific imports
from py_vollib.black_scholes import black_scholes
from py_vollib.helpers import binary_flag
from py_vollib.helpers import forward_price
from py_vollib.helpers.constants import FLOAT_MAX, MINUS_FLOAT_MAX
from py_vollib.helpers.exceptions import PriceIsAboveMaximum, PriceIsBelowIntrinsic


# -----------------------------------------------------------------------------
# FUNCTIONS

def implied_volatility(price, S, K, t, r, flag):
    """Calculate the Black-Scholes implied volatility.

    :param price: the Black-Scholes option price
    :type price: float
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str 
    
    >>> S = 100
    >>> K = 100
    >>> sigma = .2
    >>> r = .01
    >>> flag = 'c'
    >>> t = .5

    >>> price = black_scholes(flag, S, K, t, r, sigma)
    >>> iv = implied_volatility(price, S, K, t, r, flag)

    >>> expected_price = 5.87602423383
    >>> expected_iv = 0.2
    
    >>> abs(expected_price - price) < 0.00001
    True
    >>> abs(expected_iv - iv) < 0.00001
    True
    """
    deflater = numpy.exp(-r * t)
    undiscounted_option_price = price / deflater
    F = forward_price(S, t, r)
    sigma_calc = iv(undiscounted_option_price, F, K, t, binary_flag[flag])
    if sigma_calc == FLOAT_MAX:
        raise PriceIsAboveMaximum()
    elif sigma_calc == MINUS_FLOAT_MAX:
        raise PriceIsBelowIntrinsic()
    return sigma_calc


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()



# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from __future__ import print_function
import unittest
from itertools import product

# Related third party imports
import numpy

# Local application/library specific imports
from py_vollib.ref_python.black_scholes_merton.greeks.analytical import delta
from py_vollib.ref_python.black_scholes_merton.greeks.analytical import gamma
from py_vollib.ref_python.black_scholes_merton.greeks.analytical import theta
from py_vollib.ref_python.black_scholes_merton.greeks.analytical import vega
from py_vollib.ref_python.black_scholes_merton.greeks.analytical import rho
from py_vollib.ref_python.black_scholes_merton.greeks.numerical import delta as ndelta
from py_vollib.ref_python.black_scholes_merton.greeks.numerical import gamma as ngamma
from py_vollib.ref_python.black_scholes_merton.greeks.numerical import theta as ntheta
from py_vollib.ref_python.black_scholes_merton.greeks.numerical import vega as nvega
from py_vollib.ref_python.black_scholes_merton.greeks.numerical import rho as nrho


class TestRefPythonBSMGreeks(unittest.TestCase):
    def setUp(self):
        self.epsilon = 0.001
        self.flags = ['c', 'p']
        self.S = 100
        self.Ks = numpy.linspace(20, 200, 10)
        self.ts = numpy.linspace(0.01, 2, 10)
        self.rs = numpy.linspace(0, 0.2, 10)
        self.sigmas = numpy.linspace(0.1, 0.5, 10)
        self.q = 0
        self.arg_combinations = list(product(self.flags, [self.S], self.Ks, self.ts, self.rs, self.sigmas, [self.q]))

    def diff_mean(self, left, right):
        left_arr = numpy.array(left)
        right_arr = numpy.array(right)
        abs_diff = numpy.abs(left_arr - right_arr)
        return numpy.mean(abs_diff)

    def test_theta(self):
        thetas = [theta(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        nthetas = [ntheta(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        self.assertTrue(self.diff_mean(thetas, nthetas) < self.epsilon)

    def test_delta(self):
        deltas = [delta(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        ndeltas = [ndelta(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        self.assertTrue(self.diff_mean(deltas, ndeltas) < self.epsilon)

    def test_gamma(self):
        gammas = [gamma(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        ngammas = [ngamma(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        self.assertTrue(self.diff_mean(gammas, ngammas) < self.epsilon)

    def test_vega(self):
        vegas = [vega(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        nvegas = [nvega(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        self.assertTrue(self.diff_mean(vegas, nvegas) < self.epsilon)

    def test_rho(self):
        rhos = [rho(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        nrhos = [nrho(flag, S, K, t, r, sigma, q) for flag, S, K, t, r, sigma, q in self.arg_combinations]
        self.assertTrue(self.diff_mean(rhos, nrhos) < self.epsilon)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""
py_vollib.helpers.constants
~~~~~~~~~~~~~~~~~~~~~~~~~~~

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
import sys

# Related third party imports

# Local application/library specific imports


FLOAT_MAX = sys.float_info.max
MINUS_FLOAT_MAX = - FLOAT_MAX


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()



# -*- coding: utf-8 -*-

"""
py_vollib.black.implied_volatility
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from __future__ import division

# Related third party imports
import py_lets_be_rational as lets_be_rational
import numpy

# Local application/library specific imports
from py_vollib.black import black
from py_vollib.black import undiscounted_black
from py_vollib.black import normalised_black
from py_vollib.helpers import binary_flag
from py_vollib.helpers.exceptions import PriceIsAboveMaximum, PriceIsBelowIntrinsic
from py_vollib.helpers.constants import MINUS_FLOAT_MAX, FLOAT_MAX


# -----------------------------------------------------------------------------
# FUNCTIONS - IMPLIED VOLATILITY

def implied_volatility_of_discounted_option_price(discounted_option_price, F, K, r, t, flag):
    """Calculate the implied volatility of the Black option price

    :param discounted_option_price: discounted Black price of a futures option
    :type discounted_option_price: float
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param r: the risk-free interest rate
    :type r: float 
    :param t: time to expiration in years
    :type t: float
    :param flag: 'p' or 'c' for put or call
    :type flag: str

    >>> F = 100
    >>> K = 100
    >>> sigma = .2
    >>> flag = 'c'
    >>> t = .5
    >>> r = .02

    >>> discounted_call_price = black(flag, F, K, t, r, sigma)
    >>> iv = implied_volatility_of_discounted_option_price(
    ... discounted_call_price, F, K, r, t, flag)

    >>> expected_price = 5.5811067246
    >>> expected_iv = 0.2
    
    >>> abs(expected_price - discounted_call_price) < 0.00001
    True
    >>> abs(expected_iv - iv) < 0.00001
    True
    """
    
    deflater = numpy.exp(-r * t)
    undiscounted_option_price = discounted_option_price / deflater
    sigma_calc = lets_be_rational.implied_volatility_from_a_transformed_rational_guess(
        undiscounted_option_price,
        F,
        K,
        t,
        binary_flag[flag]
    )
    if sigma_calc == FLOAT_MAX:
        raise PriceIsAboveMaximum()
    elif sigma_calc == MINUS_FLOAT_MAX:
        raise PriceIsBelowIntrinsic() 
    return sigma_calc


implied_volatility = implied_volatility_of_discounted_option_price


# -----------------------------------------------------------------------------
# FUNCTIONS - IMPLIED VOLATILITY, FOR TEST & REFERENCE

def normalised_implied_volatility(beta, x, flag):
    """Calculate the normalised Black implied volatility,
    a time invariant transformation
    of Black implied volatility.

    Keyword arguments:
    
    :param x: ln(F/K) where K is the strike price, and F is the futures price
    :type x: float
    :param beta: the normalized Black price
    :type beta: float
    :param flag: 'p' or 'c' for put or call 
    :type flag: str
    
    >>> beta_call = normalised_black(0.0, 0.2, 'c')
    >>> beta_put = normalised_black(0.1,0.23232323888,'p')
    >>> normalized_b76_iv_call = normalised_implied_volatility(beta_call, 0.0, 'c')
    >>> normalized_b76_iv_put = normalised_implied_volatility(beta_put, 0.1, 'p')
    
    >>> expected_price = 0.0796556745541
    >>> expected_iv = 0.2
    
    >>> abs(expected_price - beta_call) < 0.00001
    True
    >>> abs(expected_iv - normalized_b76_iv_call) < 0.00001
    True
    
    >>> expected_price = 0.0509710222785
    >>> expected_iv = 0.23232323888
    
    >>> abs(expected_price - beta_put) < 0.00001
    True
    >>> abs(expected_iv - normalized_b76_iv_put) < 0.00001
    True
    """    

    q = binary_flag[flag]
    return lets_be_rational.normalised_implied_volatility_from_a_transformed_rational_guess(
        beta, x, q)


def normalised_implied_volatility_limited_iterations(beta, x, flag, N):
    """Calculate the normalised Black implied volatility,
    with limited iterations.

    :param x: ln(F/K) where K is the strike price, and F is the futures price
    :type x: float
    :param beta: the normalized Black price
    :type beta: float
    :param flag: 'p' or 'c' for put or call 
    :type flag: str  

    >>> beta_call = normalised_black(0.0, 0.2, 'c')
    >>> beta_put = normalised_black(0.1,0.23232323888,'p')
    >>> normalized_b76_iv_call = normalised_implied_volatility_limited_iterations(beta_call, 0.0, 'c',1)
    >>> normalized_b76_iv_put = normalised_implied_volatility_limited_iterations(beta_put, 0.1, 'p',1)
    
    >>> expected_price = 0.0796556745541
    >>> expected_iv = 0.2
    
    >>> abs(expected_price - beta_call) < 0.00001
    True
    >>> abs(expected_iv - normalized_b76_iv_call) < 0.00001
    True
    
    >>> expected_price = 0.0509710222785
    >>> expected_iv = 0.23232323888
    
    >>> abs(expected_price - beta_put) < 0.00001
    True
    >>> abs(expected_iv - normalized_b76_iv_put) < 0.00001
    True
    """    

    q = binary_flag[flag]
    return lets_be_rational.normalised_implied_volatility_from_a_transformed_rational_guess_with_limited_iterations(
        beta, x, q, N)


def implied_volatility_of_undiscounted_option_price(
        undiscounted_option_price,
        F,
        K,
        t,
        flag
    ):
    """Calculate the implied volatility of the undiscounted Black option price

    :param undiscounted_option_price: undiscounted Black price of a futures option
    :type undiscounted_option_price: float
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float  

    >>> F = 100
    >>> K = 100
    >>> sigma = .2
    >>> flag = 'c'
    >>> t = .5

    >>> undiscounted_call_price = undiscounted_black(F, K, sigma, t, flag)
    >>> iv = implied_volatility_of_undiscounted_option_price(
    ... undiscounted_call_price, F, K, t, flag)

    >>> expected_price = 5.6371977797
    >>> expected_iv = 0.2
    
    >>> abs(expected_price - undiscounted_call_price) < 0.00001
    True
    >>> abs(expected_iv - iv) < 0.00001
    True
    """

    return lets_be_rational.implied_volatility_from_a_transformed_rational_guess(
        undiscounted_option_price, 
        F,
        K, 
        t, 
        binary_flag[flag]
    )


def implied_volatility_of_undiscounted_option_price_limited_iterations(
    undiscounted_option_price, F, K, t, flag, N):
    """Calculate implied volatility of the undiscounted Black 
    option price with limited iterations.

    :param undiscounted_option_price: undiscounted Black price of a futures option
    :type undiscounted_option_price: float
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float 

    >>> F = 100
    >>> K = 100
    >>> sigma = .232323232
    >>> flag = 'c'
    >>> t = .5

    >>> price = undiscounted_black(F, K, sigma, t, flag)
    >>> iv = implied_volatility_of_undiscounted_option_price_limited_iterations(
    ... price, F, K, t, flag, 1)

    >>> expected_price = 6.54635543387
    >>> expected_iv = 0.232323232
    
    >>> abs(expected_price - price) < 0.00001
    True
    >>> abs(expected_iv - iv) < 0.00001
    True
    """  

    return lets_be_rational.implied_volatility_from_a_transformed_rational_guess_with_limited_iterations(
        undiscounted_option_price, 
        F,
        K, 
        t, 
        binary_flag[flag],
        N
    )


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()



# -*- coding: utf-8 -*-

"""
py_vollib.black
~~~~~~~~~~~~~~~

Copyright © 2017

A library for option pricing, implied volatility, and
greek calculation.  py_vollib is based on lets_be_rational,
a Python wrapper for LetsBeRational by Peter Jaeckel as
described below.

:copyright: © 2017 Gammon Capital LLC
:license: MIT, see LICENSE for more details.

About LetsBeRational:
~~~~~~~~~~~~~~~~~~~~~~~

The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

======================================================================================
Copyright © 2013-2014 Peter Jäckel.

Permission to use, copy, modify, and distribute this software is freely granted,
provided that this notice is preserved.

WARRANTY DISCLAIMER
The Software is provided "as is" without warranty of any kind, either express or implied,
including without limitation any implied warranties of condition, uninterrupted use,
merchantability, fitness for a particular purpose, or non-infringement.
======================================================================================
"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from __future__ import division
from math import log, sqrt

# Related third party imports
import py_lets_be_rational as lets_be_rational
import numpy
from numpy import log, sqrt

# Local application/library specific imports
from py_vollib.helpers import binary_flag


# -----------------------------------------------------------------------------
# FUNCTIONS - OPTION PRICING

def black(flag, F, K, t, r, sigma):
    
    """Calculate the (discounted) Black option price.

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float  
        
    >>> F = 100
    >>> K = 100
    >>> sigma = .2
    >>> flag = 'c'
    >>> r = .02
    >>> t = .5
    >>> black(flag, F, K, t, r, sigma)
    5.5811067246048118

    """
    
    deflater = numpy.exp(-r * t)
    return undiscounted_black(F, K, sigma, t, flag) * deflater

# -----------------------------------------------------------------------------
# FUNCTIONS - FOR REFERENCE AND TESTING

def undiscounted_black(F, K, sigma, t, flag):

    """Calculate the **undiscounted** Black option price.

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float   

    >>> F = 100
    >>> K = 100
    >>> sigma = .2
    >>> flag = 'c'
    >>> t = .5
    >>> undiscounted_black(F, K, sigma, t, flag)
    5.637197779701664
    
    """

    q = binary_flag[flag]
    F = float(F)
    K = float(K)
    sigma = float(sigma)
    t = float(t)
    
    return lets_be_rational.black(F, K, sigma, t, q)


def normalised_black(x, s, flag):

    """Calculate the normalised Black value,
    a "time value put-call invariant" transformation
    of the Black pricing formula.  In other words, 
    the amount of time value, or "extrinsic" value 
    of a put and call at the same log-moneyness will
    be always be identical.

    :param x: ln(F/K) where K is the strike price, and F is the futures price
    :type x: float
    :param s: volatility times the square root of time to expiration
    :type s: float
    :param flag: 'p' or 'c' for put or call 
    :type flag: str   
    
    
    >>> def normalised_intrinsic(F, K, flag): 
    ...     if flag=='c':
    ...         return max(F-K,0)/(F*K)**0.5
    ...     else:
    ...         return max(K-F,0)/(F*K)**0.5    
    
    >>> F = 100.
    >>> K = 95.
    >>> x = log(F/K)
    >>> t = 0.5
    >>> v = 0.3
    >>> s = v * sqrt(t)    
    
    >>> normalised_black(x,s,'p')
    0.061296663817558904
    
    >>> normalised_black(x,s,'c')
    0.11259558142181655
    
    '''
    Here the put is OTM, so has only time value.
    The call is ITM, having both intrinsic and time value.
    Since the time value must be equal for both, 
    the call normalised price minus its normalised 
    intrinsic must equal the put normalised price.
    
    
    >>> (normalised_black(x,s,'p') - (
    ... normalised_black(x,s,'c') - normalised_intrinsic(F,K,'c')))<1e-12 
    True
    
    
    """
    
    q = binary_flag[flag]
    
    return lets_be_rational.normalised_black(x, s, q)


if __name__ == "__main__":
    from py_vollib.helpers.doctest_helper import run_doctest
    run_doctest()





# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from __future__ import print_function
import unittest

# Related third party imports

# Local application/library specific imports
from py_vollib.ref_python.black_scholes_merton import black_scholes_merton
from py_vollib.ref_python.black_scholes_merton.implied_volatility import implied_volatility
from py_vollib.ref_python.black_scholes_merton.greeks import analytical
from py_vollib.ref_python.black_scholes_merton.greeks import numerical
from tests.test_utils import IteratorForTestData


q = 0


class TestRefPythonBlackScholesMertonAgainstBenchmarkValues(unittest.TestCase):
    def setUp(self):
        self.tdi = IteratorForTestData()

    def test_prices(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(black_scholes_merton('c', S, K, t, r, sigma, q), row['bs_call'], delta=0.000001)
            self.assertAlmostEqual(black_scholes_merton('p', S, K, t, r, sigma, q), row['bs_put'], delta=0.000001)

    def test_analytical_delta(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(analytical.delta('c', S, K, t, r, sigma, q), row['CD'], delta=0.000001)
            self.assertAlmostEqual(analytical.delta('p', S, K, t, r, sigma, q), row['PD'], delta=0.000001)

    def test_analytical_theta(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(analytical.theta('c', S, K, t, r, sigma, q), row['CT'] / 365.0, delta=0.000001)
            self.assertAlmostEqual(analytical.theta('p', S, K, t, r, sigma, q), row['PT'] / 365.0, delta=0.000001)

    def test_analytical_gamma(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(analytical.gamma('c', S, K, t, r, sigma, q), row['CG'], delta=0.000001)
            self.assertAlmostEqual(analytical.gamma('p', S, K, t, r, sigma, q), row['PG'], delta=0.000001)

    def test_analytical_vega(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(analytical.vega('c', S, K, t, r, sigma, q), row['CV'] * .01, delta=0.000001)
            self.assertAlmostEqual(analytical.vega('p', S, K, t, r, sigma, q), row['PV'] * .01, delta=0.000001)

    def test_analytical_rho(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(analytical.rho('c', S, K, t, r, sigma, q), row['CR'] * 0.01, delta=0.000000001)
            self.assertAlmostEqual(analytical.rho('p', S, K, t, r, sigma, q), row['PR'] * 0.01, delta=0.000000001)

    def test_numerical_delta(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(numerical.delta('c', S, K, t, r, sigma, q), row['CD'], delta=0.000001)
            self.assertAlmostEqual(numerical.delta('p', S, K, t, r, sigma, q), row['PD'], delta=0.000001)

    def test_numerical_theta(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(numerical.theta('c', S, K, t, r, sigma, q), row['CT'] / 365.0, delta=1)
            self.assertAlmostEqual(numerical.theta('p', S, K, t, r, sigma, q), row['PT'] / 365.0, delta=1)

    def test_numerical_gamma(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(numerical.gamma('c', S, K, t, r, sigma, q), row['CG'], delta=0.000001)
            self.assertAlmostEqual(numerical.gamma('p', S, K, t, r, sigma, q), row['PG'], delta=0.000001)

    def test_numerical_vega(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(numerical.vega('c', S, K, t, r, sigma, q), row['CV'] * 0.01, delta=0.01)
            self.assertAlmostEqual(numerical.vega('p', S, K, t, r, sigma, q), row['PV'] * 0.01, delta=0.01)

    def test_numerical_rho(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            self.assertAlmostEqual(numerical.rho('c', S, K, t, r, sigma, q), row['CR'] * 0.01, delta=0.001)
            self.assertAlmostEqual(numerical.rho('p', S, K, t, r, sigma, q), row['PR'] * 0.01, delta=0.001)

    def test_implied_volatility(self):
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S, K, t, r, sigma = row['S'], row['K'], row['t'], row['R'], row['v']
            C, P = black_scholes_merton('c', S, K, t, r, sigma, q), black_scholes_merton('p', S, K, t, r, sigma, q)
            try:
                iv = implied_volatility(C, S, K, t, r, q, 'c')
                self.assertAlmostEqual(sigma, iv, delta=0.0001)
            except:
                print('could not calculate iv for ', C, S, K, t, r, 'c')
            iv = implied_volatility(P, S, K, t, r, q, 'p')
            self.assertTrue(iv == 1e-12) if iv == 1e-12 else self.assertAlmostEqual(sigma, iv, delta=0.001)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8


from setuptools import setup, find_packages


setup(
    name='py_vollib',
    version='1.0.1',
    description='',
    url='http://vollib.org',
    maintainer='vollib',
    maintainer_email='vollib@gammoncap.com',
    license='MIT',
    install_requires=[
        'py_lets_be_rational',
        'simplejson',
        'numpy',
        'pandas',
        'scipy'
    ],
    packages=find_packages()
)
